use strict;
use warnings;

my ($in_metaeuk_fas_file, $out_exon_dist_res) = @ARGV;

open (my $out, ">", $out_exon_dist_res) or die "could not open $out_exon_dist_res for writing";
print $out "pred_id\tnum_exons\tlength_each_exon\n";
my $pred_id = 0;

open (my $in, "<", $in_metaeuk_fas_file) or die "could not open $in_metaeuk_fas_file for reading";
my $line = <$in>;
my $warning_count = 0;
while (defined $line)
{
	chomp ($line);
	if ($line =~ m/^>(.*)/)
	{
		# >ERR1719402_634740|ERR1726660_k119_1782612|+|68|9.41e-11|1|3864|4073|3864[3864]:4073[4073]:210[210]
		my ($target_id, $contig_id, $strand, $sum_bit_score, $evalue, $num_exons, $low_coord, $high_coord, @exons_info) = split(/\|/, $1);
		foreach my $exon_info (@exons_info)
		{
			#3864[3864]:4073[4073]:210[210]
			my ($start_info, $end_info, $length_info) = split(/\:/,$exon_info);
			if ($length_info =~ m/(\d+)\[(\d+)\]/)
			{
				my $before_trim_length = $1;
				my $taken_length = $2;
				if ((($before_trim_length % 3) != 0 ) or (($taken_length % 3) != 0 ))
				{
					die "Exon length is not divisible by 3: $line\n";
				}
				my $before_trim_length_aa = $before_trim_length	/ 3;
				if ($before_trim_length_aa < 9)
				{
					die "very short exon here: $line\n";
				}
				else
				{
					my $taken_length_aa = $taken_length / 3;
					if ($taken_length_aa < 9)
					{
						print "$warning_count: $line\n";
						$warning_count++;
					}
					print $out "$pred_id\t$num_exons\t$before_trim_length_aa\n";
				}
			}
		}
		$pred_id++;	
	}
	$line = <$in>;
}
close ($in);
close ($out);
