use strict;
use warnings;

my ($in_metaeuk_all_vs_targets_m8_file, $in_metaeuk_reps_fasta_file, $out_predEx_report_file, $out_predRed_report_file) = @ARGV;

my %metaeuk_reps;
collect_metaeuk_reps($in_metaeuk_reps_fasta_file, \%metaeuk_reps);

open (my $out_all, ">", $out_predEx_report_file) or die "could not open $out_predEx_report_file for writing\n";
open (my $out_red, ">", $out_predRed_report_file) or die "could not open $out_predRed_report_file for writing\n";
print $out_all "metaeuk_TCS\tis_rep\tseqid_to_T\tqcov\ttcov\tsw_bits\tsw_evalue\tmetaeuk_bits\tmetaeuk_evalue\tmetaeuk_num_exons\n";
print $out_red "metaeuk_TCS\tis_rep\tseqid_to_T\tqcov\ttcov\tsw_bits\tsw_evalue\tmetaeuk_bits\tmetaeuk_evalue\tmetaeuk_num_exons\n";
open (my $in, "<", $in_metaeuk_all_vs_targets_m8_file) or die "could not open $in_metaeuk_all_vs_targets_m8_file for reading\n";
my $m8_line = <$in>;
while (defined $m8_line)
{
	chomp($m8_line);
	# 414855|NW_004457733.1|-|56|6.585e-07|1|16812|17207|17207[17207]:16812[16812]:396[396]	UniRef90_R7QP01	4.627E-08	0.319	1.000	0.255	141	51	4
	my ($metaeuk_pred_header, $target_acc, $evalue, $pident, $qcov, $tcov, $alnlen, $mismatch, $gapopen, $bits) = split(/\t/, $m8_line);
	my $metaeuk_TCS = get_TCS_key($metaeuk_pred_header, $target_acc);
	my @header_parts = split(/\|/, $metaeuk_pred_header);
	my ($target_id, $contig_id, $strand, $sum_bit_score, $metaeuk_evalue, $num_exons, $low_coord, $high_coord, @exons_info) = @header_parts;
	
	my $is_rep = "no";
	if (exists $metaeuk_reps{$metaeuk_TCS}) {
		$is_rep = "yes";
		print $out_red "$metaeuk_TCS\t$is_rep\t$pident\t$qcov\t$tcov\t$bits\t$evalue\t$sum_bit_score\t$metaeuk_evalue\t$num_exons\n";
	}

	print $out_all "$metaeuk_TCS\t$is_rep\t$pident\t$qcov\t$tcov\t$bits\t$evalue\t$sum_bit_score\t$metaeuk_evalue\t$num_exons\n";
	
	$m8_line = <$in>;
}
close ($in);
close ($out_all);
close ($out_red);

sub collect_metaeuk_reps {
	my ($in_fasta_file, $metaeuk_reps_ref) = @_;
	open (my $in, "<", $in_fasta_file) or die "could not open $in_fasta_file for reading\n";
	my $line = <$in>;
	while (defined $line) {
		chomp $line;
		if ($line =~ m/^>(.*)/) {
			my $header = $1;
			my @header_parts = split(/\|/, $header);
			my ($target_id, $contig_id, $strand, $sum_bit_score, $evalue, $num_exons, $low_coord, $high_coord, @exons_info) = @header_parts;
			my $key = get_TCS_key($header);
			$metaeuk_reps_ref->{$key} = 1;
		}
		$line = <$in>;
	}
	close ($in);
}

sub get_TCS_key {
	my ($header, $target_acc) = @_;
	my @header_parts = split(/\|/, $header);
	my ($target_id, $contig_id, $strand, $sum_bit_score, $evalue, $num_exons, $low_coord, $high_coord, @exons_info) = @header_parts;
	my $TCS_key = join("\|", ($target_id, $contig_id, $strand));
	if (defined $target_acc) {
		$TCS_key = join("\|", ($target_acc, $contig_id, $strand));
	}
	return($TCS_key);
}