use strict;
use warnings;

#@ARGV = ("predClust.tsv", sw_predEx_to_target.m8");
my ($in_metaeuk_preds_to_targets_m8_file, $in_metaeuk_reps_to_members_file, $out_reps_to_min_seqid_file) = @ARGV;

my %preds_to_targets;
read_pred_to_target_m8($in_metaeuk_preds_to_targets_m8_file, \%preds_to_targets);

my %rep_to_min_seqid;
rep_to_group_min_seqid($in_metaeuk_reps_to_members_file, \%preds_to_targets, \%rep_to_min_seqid);

open (my $out, ">", $out_reps_to_min_seqid_file) or die "could not open $out_reps_to_min_seqid_file for writing\n";
foreach my $TCS_rep (keys %rep_to_min_seqid) {
	print $out "$TCS_rep\t$rep_to_min_seqid{$TCS_rep}\n";
}
close ($out);

sub read_pred_to_target_m8 {
	my ($in_metaeuk_preds_to_targets_m8_file, $preds_to_targets_ref) = @_;
	
	open (my $in, "<", $in_metaeuk_preds_to_targets_m8_file) or die "could not open $in_metaeuk_preds_to_targets_m8_file for reading\n";
	my $line = <$in>;
	while (defined $line) {
		#309|NC_003424.3|+|869|1.205e-251|1|2932856|2934910|2932856[2932856]:2934910[2934910]:2055[2055]	UniRef90_P12807	3.924E-282	0.587	1.000	0.973	686	269	4
		chomp ($line);
		my ($metaeuk_header, $targetAcc, $evalue, $pident, $qcov, $tcov, $alnlen, $mismatch, $gapopen) = split(/\t/, $line);
		my ($targetKey, $contigAcc, $strand) = split(/\|/, $metaeuk_header);
		my $TCS_str = join("\|", ($targetAcc, $contigAcc, $strand));
		
		if (! exists $preds_to_targets_ref->{$TCS_str}) {
			$preds_to_targets_ref->{$TCS_str} = {"evalue" => $evalue, "seqid" => $pident, "metaeuk_cov" => $qcov, "target_cov" => $tcov, 
												 "alnlen" => $alnlen, "mismatch" => $mismatch, "gapopen" => $gapopen};
		} else {
			die "$TCS_str appears twice in result file...:\n$line\n";
		}
		$line = <$in>;
	}
	close ($in);
}

sub rep_to_group_min_seqid {
	my ($in_metaeuk_reps_to_members_file, $preds_to_targets_ref, $rep_to_min_seqid_ref) = @_;
	
	open (my $in, "<", $in_metaeuk_reps_to_members_file) or die "could not open $in_metaeuk_reps_to_members_file for reading\n";
	my $line = <$in>;
	while (defined $line) {
		#UniRef90_UPI000689236A|NC_003424.3|+	UniRef90_UPI000689236A|NC_003424.3|+
		#UniRef90_UPI000689236A|NC_003424.3|+	UniRef90_A0A0C3BVX7|NC_003424.3|+
		chomp ($line);
		my ($TCS_rep, $TCS_mem) = split(/\t/, $line);
		
		if (! exists $preds_to_targets_ref->{$TCS_mem}) {
			die "Problematic line: $line\n. Could not find $TCS_mem in pred_to_target.m8\n";
		}
		
		if (exists $rep_to_min_seqid_ref->{$TCS_rep}) {
			if ($rep_to_min_seqid_ref->{$TCS_rep} > $preds_to_targets_ref->{$TCS_mem}{"seqid"}) {
				$rep_to_min_seqid_ref->{$TCS_rep} = $preds_to_targets_ref->{$TCS_mem}{"seqid"};
			}
		} else {
			$rep_to_min_seqid_ref->{$TCS_rep} = $preds_to_targets_ref->{$TCS_mem}{"seqid"};
		}
		
		$line = <$in>;
	}
	close ($in);
}
