#!/bin/bash -ex
#SBATCH -J Tax_preds
#SBATCH -p hh
#SBATCH -o out.%J
#SBATCH -n 160
#SBATCH -N 10
#SBATCH -t 14-00:00:00
export RUNNER="mpirun --pernode --bind-to none"

MMSEQS_PATH="YOUR_DIR/mmseqs2_790d/MMseqs2/build/src/mmseqs"
SCRIPTS_DIR="scripts"
METAEUK_RESULTS_OUTDIR="TaraEukRep5kb_EUK_profiles_011"
METAEUK_PREDS_FASTA="${METAEUK_RESULTS_OUTDIR}/predRed_Tara5kbEukRep.fas"
TAX_SUBDIR="${METAEUK_RESULTS_OUTDIR}/taxonomy"

MMETSP_ID_TO_NAME_AND_LINEAGE="misc/MMETSP_IDs_to_names_and_lineage_str.txt"

declare -a TAX_INFO_DIRS=("SEE_RELEASED_DATA/TAX_DBs/UNICLUST/" "SEE_RELEASED_DATA/TAX_DBs/MMETSP/");
declare -a TAXDBNAMES=("uniclust90_2018_08_seed" "MMETSP");

mkdir -p "${TAX_SUBDIR}"
cd "${TAX_SUBDIR}"
"${MMSEQS_PATH}" createdb "${METAEUK_PREDS_FASTA}" predRed --dbtype 1

for i in "${!TAX_INFO_DIRS[@]}"; do
	cd "${TAX_SUBDIR}"
	TAX_INFO_DIR="${TAX_INFO_DIRS[$i]}"
	TAXDBNAME="${TAXDBNAMES[$i]}"
	
	# run taxonomy
	if [ ! -f "${TAX_SUBDIR}/${TAXDBNAME}_LCA_Tara5kbEukRep.dbtype" ]
	then
		"${MMSEQS_PATH}" taxonomy "${TAX_SUBDIR}/predRed" "${TAX_INFO_DIR}/${TAXDBNAME}" "${TAX_SUBDIR}/${TAXDBNAME}_LCA_Tara5kbEukRep" "${TAXDBNAME}_tmpTax" --start-sens 2 --sens-steps 3 -s 6
	fi

	# extract the best taxonomic hit of each prediction (simpler than 2bLCA)
	cd "${TAX_SUBDIR}/${TAXDBNAME}_tmpTax/latest/"
	if [ ! -f "preds_best_hit_tax.dbtype" ]
	then
		"${MMSEQS_PATH}" filterdb --extract-lines 1 first first_extracted
		"${MMSEQS_PATH}" lca "${TAX_INFO_DIR}/${TAXDBNAME}" first_extracted preds_best_hit_tax --lca-ranks "species:genus:family:order:class:phylum:superphylum:subkingdom:kingdom:superkingdom"
	fi
	"${MMSEQS_PATH}" createtsv "${TAX_SUBDIR}/predRed" preds_best_hit_tax "${TAX_SUBDIR}/taxRep_${TAXDBNAME}_BestHit_Tara5kbEukRep.tsv"

	# extract the score of each prediction against its best taxonomic hit #
	"${MMSEQS_PATH}" convertalis "${TAX_SUBDIR}/predRed" "${TAX_INFO_DIR}/${TAXDBNAME}" first_extracted "${TAX_SUBDIR}/taxRep_${TAXDBNAME}_BestHit_Tara5kbEukRep.m8"

	# join the taxonomic and score info #
	awk 'NR == FNR { f[$1] = $11"\t"$12"\t"$3"\t"$2; next } $1 in f { print $0"\t"f[$1] }' "${TAX_SUBDIR}/taxRep_${TAXDBNAME}_BestHit_Tara5kbEukRep.m8" "${TAX_SUBDIR}/taxRep_${TAXDBNAME}_BestHit_Tara5kbEukRep.tsv" > "${TAX_SUBDIR}/taxRep_${TAXDBNAME}_BestHit_Tara5kbEukRep_evalue_bitscore.tsv"
done

# collect the best hit from the two sources #
perl "${SCRIPTS_DIR}"/assign_tax_to_pred_and_contig.pl "${METAEUK_PREDS_FASTA}" "${MMETSP_ID_TO_NAME_AND_LINEAGE}" "${TAX_SUBDIR}"/taxRep_"${TAXDBNAMES[0]}"_BestHit_Tara5kbEukRep_evalue_bitscore.tsv "${TAX_SUBDIR}"/taxRep_"${TAXDBNAMES[1]}"_BestHit_Tara5kbEukRep_evalue_bitscore.tsv "${METAEUK_RESULTS_OUTDIR}"/joinedTaxBestHit_Tara5kbEukRep_preds_all_ranks.tsv "${METAEUK_RESULTS_OUTDIR}"/joinedTaxBestHit_Tara5kbEukRep_contigs_all_ranks.tsv "${METAEUK_RESULTS_OUTDIR}"/joinedTaxBestHit_Tara5kbEukRep_contigs_for_corona.tsv

# assign phylum+ to contigs: #
perl "${SCRIPTS_DIR}"/assign_supergroup_to_contig.pl "${METAEUK_RESULTS_OUTDIR}"/joinedTaxBestHit_Tara5kbEukRep_contigs_all_ranks.tsv "${METAEUK_RESULTS_OUTDIR}"/joinedTaxBestHit_Tara5kbEukRep_contigs_all_ranks_for_R.tsv
