#!/bin/bash -ex
#SBATCH -J MetaEUK_Tara
#SBATCH -p hh
#SBATCH -o out.%J
#SBATCH -n 160
#SBATCH -N 10
#SBATCH -t 14-00:00:00
export RUNNER="mpirun --pernode --bind-to none"

METAEUK_DIR="YOUR_DIR/mmseqs_ebb1_metaeuk_4714"
METAEUK_PATH="YOUR_DIR/mmseqs_ebb1_metaeuk_4714/build/src/metaeuk"
OUTDIR="TaraEukRep5kb_EUK_profiles_011"
EUK_PROFILES="SEE_RELEASED_DATA/EukRefs/linclusted/MMETSP_uniclust50_MERC_profiles"
TARA_EUKREP_5KB_CONTIGS_FASTA="SEE_RELEASED_DATA/assembled_atlas_5kb_EukRep_contigs.fas"
LOCAL_TMP_PATH="/local/"

mkdir -p "${OUTDIR}"

# create DB from Tara contigs fasta #
cd "${OUTDIR}"
if [ ! -f "Tara_contigs" ]
then
	"${METAEUK_PATH}" createdb "${TARA_EUKREP_5kb_CONTIGS_FASTA}" Tara_contigs --dont-split-seq-by-len --dbtype 2
fi

# run metaeuk (record mmseqs2 and metaeuk versions, measure time)#
cd "${METAEUK_DIR}";
version_mmseqs2=`git submodule status`;
version_metaeuk=`git log -1 --format="%H"`;
cd "${OUTDIR}"
echo "version_mmseqs2: $version_mmseqs2, version_metaeuk: $version_metaeuk" > versions_mmseqs2_metaeuk.txt;

touch _time_start_.txt

# predict:
"${METAEUK_PATH}" predictexons Tara_contigs "${EUK_PROFILES}" predEx_Tara5kbEukRep tempFolder --metaeuk-eval 0.0001 -e 100 --slice-search --min-ungapped-score 35 --min-length 40 --split-memory-limit 80G --min-exon-aa 20 --metaeuk-tcov 0.6 --local-tmp "${LOCAL_TMP_PATH}" --disk-space-limit 300G;
# reduce redundancy:
"${METAEUK_PATH}" reduceredundancy predEx_Tara5kbEukRep predRed_Tara5kbEukRep predClust_Tara5kbEukRep
# unite reduced predictions in a fasta:
"${METAEUK_PATH}" unitesetstofasta Tara_contigs "${EUK_PROFILES}" predRed_Tara5kbEukRep predRed_Tara5kbEukRep.fas --protein 1

touch _time_end_.txt

