# THIS ANALYSIS REQUIRES THE PFAM PROFILES DATABASE, WHICH IS NOT INCLUDED IN THE RELEASE
# WE PROVIDE THE OUTPUT OF THIS ANALYSIS AS PART OF THE RNA POL TREE BUNDLE
# IF YOU ARE INTERESTED IN THIS RUN, PLEASE CONTACT THE AUTHORS DIRECTLY...


#!/bin/bash -ex
#BSUB -J preds_vs_pfam
#SBATCH -p hh
#SBATCH -o out.%J
#SBATCH -n 16
#SBATCH -N 1
#SBATCH -t 14-00:00:00
export RUNNER="mpirun --pernode --bind-to none"

MMSEQS_PATH="YOUR_DIR/mmseqs2_790d/MMseqs2/build/src/mmseqs"
SCRIPTS_DIR="YOUR_DIR/scripts"

METAEUK_RESULTS_OUTDIR="TaraEukRep5kb_EUK_profiles_011"
METAEUK_PREDS_FASTA="${METAEUK_RESULTS_OUTDIR}/predRed_Tara5kbEukRep.fas"

# Pfam info:
PFAM_PROFILES="NOT_RELEASED/pfam_profiles"
OG5_127924_PFAM_DOMAINS="misc/OG5_127924_PFAM_domains.txt"

# output
OG5_127924_OUTDIR="${METAEUK_RESULTS_OUTDIR}/OG5_127924_ANALYSIS"

mkdir -p "${OG5_127924_OUTDIR}"
cd "${OG5_127924_OUTDIR}"
"${MMSEQS_PATH}" createdb "${METAEUK_PREDS_FASTA}" predRed --dbtype 1

# get TaraEukRep5kbPreds matches to all 5 domains in the right order #
if [ ! -f MetaEuk_preds_vs_PFAM.m8 ]
then
	"${MMSEQS_PATH}" search predRed "${PFAM_PROFILES}" MetaEuk_preds_vs_PFAM tmp -k 5 -s 7 -a
	"${MMSEQS_PATH}" convertalis predRed "${PFAM_PROFILES}" MetaEuk_preds_vs_PFAM MetaEuk_preds_vs_PFAM.m8
fi

perl "${SCRIPTS_DIR}"/get_multi_match.pl MetaEuk_preds_vs_PFAM.m8 "${OG5_127924_PFAM_DOMAINS}" 0.0001 TaraEukRep5kbPreds_multi_match_OG5_127924_PFAM_domains.m8 TaraEukRep5kbPreds_multi_match_OG5_127924_PFAM_domains_headers.txt

# create a fasta of these TaraEukRep5kbPreds #
perl "${SCRIPTS_DIR}"/get_subset_from_fasta.pl "${METAEUK_PREDS_FASTA}" TaraEukRep5kbPreds_multi_match_OG5_127924_PFAM_domains_headers.txt TaraEukRep5kbPreds_multi_match_OG5_127924_PFAM_domains.fas

#### shorten and map ####
perl "${SCRIPTS_DIR}"/shorten_metaeuk_headers_and_map.pl TaraEukRep5kbPreds_multi_match_OG5_127924_PFAM_domains.fas TaraEukRep5kbPreds_multi_match_OG5_127924_PFAM_domains_short_heads.fas TaraEukRep5kbPreds_multi_match_OG5_127924_PFAM_domains_short_heads_map.txt

#### cluster duplicates ####
if [ ! -f TaraEukRep5kbPreds_multi_match_OG5_127924_PFAM_domains_short_heads_no_dups.fas ]
then
	CLU_OUTDIR="${OG5_127924_OUTDIR}/no_dups"
	mkdir -p "${CLU_OUTDIR}"
	cd "${CLU_OUTDIR}"

	"${MMSEQS_PATH}" createdb ../TaraEukRep5kbPreds_multi_match_OG5_127924_PFAM_domains_short_heads.fas metaeuk_OG5_127924 --dont-split-seq-by-len --dbtype 1
	"${MMSEQS_PATH}" cluster metaeuk_OG5_127924 metaeuk_OG5_127924_clu --min-seq-id 1 tmp
	"${MMSEQS_PATH}" result2repseq metaeuk_OG5_127924 metaeuk_OG5_127924_clu metaeuk_OG5_127924_clu_rep
	"${MMSEQS_PATH}" result2flat metaeuk_OG5_127924 metaeuk_OG5_127924 metaeuk_OG5_127924_clu_rep ../TaraEukRep5kbPreds_multi_match_OG5_127924_PFAM_domains_short_heads_no_dups.fas --use-fasta-header
fi

