library(ggplot2)
library(scales)
library(plyr)
library(grid)
library(gridExtra)

# define default font sizes, etc.
theme_set(theme_classic(
  base_size = 11
))

args <- commandArgs(TRUE)
supergroups_of_contigs_file <- as.character(args[1])
path_for_plot <- as.character(args[2])

## for debug:
supergroups_of_contigs_file="tara/run_011/joinedTaxBestHit_Tara5kbEukRep_contigs_all_ranks_for_R.tsv"
path_for_plot="tara/run_011/plots/joinedTaxBestHit_Tara5kbEukRep_contigs_with_preds"

dir.create(file.path("tara/run_011/plots/"), showWarnings = FALSE)

path_for_plot_png = paste(path_for_plot,".png",sep = "")
path_for_plot_pdf = paste(path_for_plot,".pdf",sep = "")

supergroups_of_contigs = read.delim(file = supergroups_of_contigs_file, header = TRUE, sep = "\t")

# add number of predictions category #
supergroups_of_contigs$NUM_PREDS_CAT = "NA"
supergroups_of_contigs$NUM_PREDS_CAT[which(supergroups_of_contigs$NUM_PREDS <= 2)] = "1-2"
supergroups_of_contigs$NUM_PREDS_CAT[which((supergroups_of_contigs$NUM_PREDS >= 3) & (supergroups_of_contigs$NUM_PREDS <= 5))] = "3-5"
supergroups_of_contigs$NUM_PREDS_CAT[which((supergroups_of_contigs$NUM_PREDS >= 6) & (supergroups_of_contigs$NUM_PREDS <= 10))] = "6-10"
supergroups_of_contigs$NUM_PREDS_CAT[which(supergroups_of_contigs$NUM_PREDS > 10)] = "10+"

# sum counts per domain and num pred category #
supergroups_of_contigs$DOMAIN = factor(supergroups_of_contigs$DOMAIN, levels=c("eukaryota", "archaea", "bacteria", "viruses", "no_match"), labels=c("Eukaryota", "Archaea", "Bacteria", "Viruses", "Unassigned"))
counts_of_domains_by_num_preds_cat = table(supergroups_of_contigs$DOMAIN,supergroups_of_contigs$NUM_PREDS_CAT)
counts_of_domains_by_num_preds_cat_df = as.data.frame(counts_of_domains_by_num_preds_cat)
colnames(counts_of_domains_by_num_preds_cat_df) = c("DOMAIN","NUM_PREDS_CAT","CONTIG_COUNT")
counts_of_domains_by_num_preds_cat_df$PERCENT = round(100 * counts_of_domains_by_num_preds_cat_df$CONTIG_COUNT / sum(counts_of_domains_by_num_preds_cat_df$CONTIG_COUNT), digits = 1)
counts_of_domains_by_num_preds_cat_df$NUM_PREDS_CAT = factor(counts_of_domains_by_num_preds_cat_df$NUM_PREDS_CAT, levels=c("1-2", "3-5", "6-10", "10+"))
counts_of_domains_by_num_preds_cat_df = counts_of_domains_by_num_preds_cat_df[order(counts_of_domains_by_num_preds_cat_df$DOMAIN,counts_of_domains_by_num_preds_cat_df$PERCENT),]

# compute the percent of each domain in each contig num category
counts_of_domains_by_num_preds_cat_df$PERCENT_OF_CAT = 0
for (num_preds_cat in unique(counts_of_domains_by_num_preds_cat_df$NUM_PREDS_CAT)) {
  curr_inds = which(counts_of_domains_by_num_preds_cat_df$NUM_PREDS_CAT == num_preds_cat &
                      counts_of_domains_by_num_preds_cat_df$DOMAIN != "Unassigned")
  total_assigned_contigs = sum(counts_of_domains_by_num_preds_cat_df$CONTIG_COUNT[curr_inds])
  counts_of_domains_by_num_preds_cat_df$PERCENT_OF_CAT[curr_inds] = counts_of_domains_by_num_preds_cat_df$CONTIG_COUNT[curr_inds] / total_assigned_contigs
}

domain_colors = c("brown1", "#FFCCE5", "#43464B", "#b2df8a","gray68")
fig_contigs_domains_by_num_preds = ggplot(counts_of_domains_by_num_preds_cat_df, aes(x=NUM_PREDS_CAT, y=PERCENT, fill=rev(DOMAIN))) +
  geom_bar(stat="identity")  +
  scale_fill_manual("", values = rev(domain_colors), breaks=rev(levels(supergroups_of_contigs$DOMAIN)), 
                    labels=(levels(supergroups_of_contigs$DOMAIN))) + 
  ylab("Contigs") +
  xlab("# preds.") +
  scale_y_continuous(breaks = seq(10,40,by=10), limits = c(0,100), labels = scales::percent(seq(0.1, 0.4, by = 0.1), accuracy = 1), expand = c(0,0)) +
  theme(
    legend.position = c(0.4,0.8)
  ) +
  geom_text(aes(label = ifelse(counts_of_domains_by_num_preds_cat_df$PERCENT_OF_CAT > 0.85, scales::percent(counts_of_domains_by_num_preds_cat_df$PERCENT_OF_CAT, accuracy = 0.1), "")), size=3.5, position=position_stack(vjust = 0.5))

##########

# sum counts just for eukaryotic supergroups #
supergroups_of_contigs_domain_counts_zero = supergroups_of_contigs[which(supergroups_of_contigs$DOMAIN == "Eukaryota"),]
supergroups_of_contigs_domain_counts_zero$SUPERGROUP = as.character(supergroups_of_contigs_domain_counts_zero$SUPERGROUP)

counts_of_euk_supergroups = table(supergroups_of_contigs_domain_counts_zero$SUPERGROUP)
counts_of_euk_supergroups_df = as.data.frame(counts_of_euk_supergroups)
colnames(counts_of_euk_supergroups_df) = c("EUK_SUPERGROUP","CONTIG_COUNT")

total_number_of_euk_contigs = sum(counts_of_domains_by_num_preds_cat_df$CONTIG_COUNT[which(counts_of_domains_by_num_preds_cat_df$DOMAIN == "Eukaryota")])
counts_of_euk_supergroups_df$PERCENT = round(100 * counts_of_euk_supergroups_df$CONTIG_COUNT / total_number_of_euk_contigs, digits = 1)
counts_of_euk_supergroups_df$PERCENT_LABEL = percent(counts_of_euk_supergroups_df$PERCENT / 100)
counts_of_euk_supergroups_df$PERCENT_LABEL[which(counts_of_euk_supergroups_df$PERCENT == 0)] = ""

counts_of_euk_supergroups_df = counts_of_euk_supergroups_df[order(counts_of_euk_supergroups_df$CONTIG_COUNT),]

fig_contigs_euk_common_supergroups = ggplot(counts_of_euk_supergroups_df, aes(y = log(CONTIG_COUNT), x = reorder(EUK_SUPERGROUP, log(CONTIG_COUNT)))) + 
  geom_bar(stat="identity", position = "dodge", width=0.75, fill = "brown1", alpha = 0.8) +
  geom_text(aes(label = PERCENT_LABEL, x = EUK_SUPERGROUP), position = position_dodge(width = 0.8), hjust = 1.2, vjust = 0.5, size = 3.5) +
  scale_y_continuous(breaks = log(c(1,5,10,1000,5000,200000)-1), labels = comma(c(1,5,10,1000,5000,200000)), expand = c(0,0)) +
  xlab("Eukaryotic supergroups") + 
  ylab("# eukaryotic contigs") + 
  coord_flip()

#####

panel_A = fig_contigs_domains_by_num_preds + 
  coord_flip() + 
  scale_x_discrete(expand = c(0,0)) + 
  theme(legend.position = c(0.45,0.8)) +
  guides(fill=guide_legend(ncol=2))
  

path_for_plot_panelA_png = paste(path_for_plot,"_A.png",sep = "")
path_for_plot_panelA_pdf = paste(path_for_plot,"_A.pdf",sep = "")

ggsave(path_for_plot_panelA_png, plot = panel_A, width = 2*150, height = 2*20, units = c("mm"))
ggsave(path_for_plot_panelA_pdf, plot = panel_A, width = 2*150, height = 2*20, units = c("mm"))

#####

FIG_contigs_taxonomy = grid.arrange(grobs = list(arrangeGrob(fig_contigs_domains_by_num_preds,
                                                              top = textGrob("A", x = unit(0.05, "npc"), y = unit(0.7, "npc"), just=c("left","top"), gp=gpar(fontface="bold", col="black", fontsize=14)),
                                                              ncol=1,nrow=1),
                                                  arrangeGrob(fig_contigs_euk_common_supergroups,
                                                              top = textGrob("B", x = unit(0.05, "npc"), y = unit(0.7, "npc"), just=c("left","top"), gp=gpar(fontface="bold", col="black", fontsize=14)),
                                                              ncol=1,nrow=1)),
                                     layout_matrix = rbind(c(1,2,2)))

ggsave(path_for_plot_png, plot = FIG_contigs_taxonomy, width = 170, height = 85, units = c("mm"))
ggsave(path_for_plot_pdf, plot = FIG_contigs_taxonomy, width = 170, height = 85, units = c("mm"))

