use strict;
use warnings;

my ($in_contigs_fasta, $in_preds_fasta, $out_per_contig_report) = @ARGV;

my %contigs_hash;

open (my $in, "<", $in_contigs_fasta) or die "could not open $in_contigs_fasta for reading";

my $line = <$in>;
my $num_contigs = 0;
while (defined $line)
{
	chomp ($line);
	if ($line =~ m/^>(.*)/)
	{
		my $contig_id = $1;
		my $seq = "";
		$line = <$in>;
		while ((defined $line) and ($line !~ m/^>/))
		{
			chomp ($line);
			$seq = $seq . $line;
			$line = <$in>;
		}
		my $contig_length = length($seq);
		$contigs_hash{$contig_id}->{"length"} = $contig_length;
		$contigs_hash{$contig_id}->{"num_preds"} = 0;
		$contigs_hash{$contig_id}->{"num_preds_plus"} = 0;
		$contigs_hash{$contig_id}->{"num_preds_minus"} = 0;
		$contigs_hash{$contig_id}->{"preds_plus_str"} = "";
		$contigs_hash{$contig_id}->{"preds_minus_str"} = "";

		if ($num_contigs % 1000 == 0)
		{
			print "entered '$contig_id' with $contig_length\n";
		}
		$num_contigs++;
	}
}
close ($in);

print "\n\nentered $num_contigs contigs\n\n";

open (my $in_preds, "<", $in_preds_fasta) or die "could not open $in_preds_fasta for reading";

$line = <$in_preds>;

my $num_preds = 0;

while (defined $line)
{
	chomp ($line);
	#>ERR1711860_51194|ERR1726599_k119_1821880|-|160|1.9e-38|1|3364|3630|3630[3630]:3364[3364]:267[267]
	if ($line =~ m/^>(.*)/)
	{
		my $metaeuk_pred_header = $1;
		my ($target_id, $contig_id, $strand, $sum_bit_score, $evalue, $num_exons, $low_coord, $high_coord, @exons_info) = split(/\|/, $metaeuk_pred_header);
		$line = <$in_preds>;
		while ((defined $line) and ($line !~ m/^>/))
		{
			$line = <$in_preds>;
		}
		if (! exists $contigs_hash{$contig_id})
		{
			die "could not find '$contig_id' in contigs input!!!\n";
		}
		else
		{
			if ($strand eq '+')
			{
				$contigs_hash{$contig_id}->{"num_preds_plus"} = $contigs_hash{$contig_id}->{"num_preds_plus"} + 1;
				$contigs_hash{$contig_id}->{"preds_plus_str"} = $contigs_hash{$contig_id}->{"preds_plus_str"} . "$low_coord\-$high_coord\,";
			}
			else
			{
				$contigs_hash{$contig_id}->{"num_preds_minus"} = $contigs_hash{$contig_id}->{"num_preds_minus"} + 1;
				$contigs_hash{$contig_id}->{"preds_minus_str"} = $contigs_hash{$contig_id}->{"preds_minus_str"} . "$low_coord\-$high_coord\,";
			}
			
			$contigs_hash{$contig_id}->{"num_preds"} = $contigs_hash{$contig_id}->{"num_preds_plus"} + $contigs_hash{$contig_id}->{"num_preds_minus"};
			
		}
		if ($num_preds % 10000 == 0)
		{
			print "entered $num_preds preds thus far\n";
		}
		$num_preds++;
	}
}

close ($in_preds);

print "\n\nentered $num_preds in total\n\n";

open (my $out, ">", $out_per_contig_report) or die "could not open $out_per_contig_report for writing";

print $out "CONTIG_ID\tLENGTH\tNUM_PREDS\tNUM_PLUS\tNUM_MINUS\tPLUS_COORDS\tMINUS_COORDS\n";

foreach my $contig_id (keys %contigs_hash)
{
	my $len = $contigs_hash{$contig_id}->{"length"};
	my $num_preds = $contigs_hash{$contig_id}->{"num_preds"};
	my $num_preds_plus = $contigs_hash{$contig_id}->{"num_preds_plus"};
	my $num_preds_minus = $contigs_hash{$contig_id}->{"num_preds_minus"};
	my $preds_plus_str = $contigs_hash{$contig_id}->{"preds_plus_str"};
	chop($preds_plus_str); # remove ','
	my $preds_minus_str = $contigs_hash{$contig_id}->{"preds_minus_str"};
	chop($preds_minus_str); # remove ','
	
	print $out "$contig_id\t$len\t$num_preds\t$num_preds_plus\t$num_preds_minus\t$preds_plus_str\t$preds_minus_str\n";
}

close ($out);
