use strict;
use warnings;

my ($in_protein_table, $in_proteomics_fasta, $in_genomics_fasta, $out_report) = @ARGV;

my %contig_to_prot;
parse_tab_NCBI_proteome($in_protein_table,\%contig_to_prot);

my %genomics_contigs;
get_ids_from_fasta($in_genomics_fasta, \%genomics_contigs);

my %protein_ids;
get_ids_from_fasta($in_proteomics_fasta, \%protein_ids);

open (my $out, ">", $out_report) or die "could not open $out_report for writing";
my $num_present_contigs = 0;
my $total_num_contigs = 0;
foreach my $contig (keys %contig_to_prot)
{
	if (! exists $genomics_contigs{$contig})
	{
		print $out "Contig '$contig' is missing from $in_genomics_fasta\n";
	}
	else
	{
		$num_present_contigs++;
	}
	$total_num_contigs++;
}
print $out "$num_present_contigs \/ $total_num_contigs were found in $in_genomics_fasta\n\n";


my $num_from_table_present_proteins = 0;
my $total_num_proteins_in_table = 0;
foreach my $contig (keys %contig_to_prot)
{
	foreach my $protein (keys %{$contig_to_prot{$contig}})
	{
		if (! exists $protein_ids{$protein})
		{
			print $out "Protein '$protein' is missing from $in_proteomics_fasta\n";
		}
		else
		{
			$num_from_table_present_proteins++;
		}
		$total_num_proteins_in_table++;
	}
}
print $out "$num_from_table_present_proteins \/ $total_num_proteins_in_table were found in $in_proteomics_fasta\n\n";


my $num_from_proteomics_present_proteins = 0;
my $total_num_proteins_in_proteomics = 0;
foreach my $protein (keys %protein_ids)
{
	my $exist_somewhere = 'no';
	foreach my $contig (keys %contig_to_prot)
	{
		if (exists $contig_to_prot{$contig}->{$protein})
		{
			$exist_somewhere = 'yes';
			$num_from_proteomics_present_proteins++;
			last;
		}
	}
	$total_num_proteins_in_proteomics++;
	if ($exist_somewhere eq 'no')
	{
		print $out "Protein '$protein' is missing from $in_protein_table\n";
	}
}
print $out "$num_from_proteomics_present_proteins \/ $total_num_proteins_in_proteomics were found in $in_protein_table\n\n";




sub get_ids_from_fasta
{
	my ($in_fasta, $ids_ref) = @_;
	open (my $in, "<", $in_fasta) or die "could not open $in_fasta for reading";
	my $line = <$in>;
	
	while (defined $line)
	{
		chomp ($line);
		if ($line =~ m/^>(.*)/)
		{
			$ids_ref->{$1} = 1;
		}
		$line = <$in>;
	}
	
	close ($in);
}


sub parse_tab_NCBI_proteome
{
	my ($in_tab_delim_NCBI_prot, $contig_to_prot_ref) = @_;
	open (my $in, "<", $in_tab_delim_NCBI_prot) or die "could not open $in_tab_delim_NCBI_prot for reading";
	
	print "starting to fill the NCBI hash\n";
	### These tables are irregular in structure... E.g.:###
	#Replicon_Name	Replicon_Accession	Start	Stop	Strand	geneID	Locus	Locus_tag	Protein_product	Length	Protein name
	#Replicon Name	Replicon Accession	Start	Stop	Strand	GeneID	Locus tag	Protein product	Length	Protein name
	#Replicon Name	Replicon Accession	Start	Stop	Strand	GeneID	Locus	Locus tag	Protein product	Length	Protein name
		
	my $header = <$in>;
	chomp ($header);
	
	my @header_parts = split(/\t/, $header);
	foreach my $header_part (@header_parts)
	{
		print "Header part: '$header_part'\n";
	}
	
	my $ind_contig_id = 'NA';
	my $ind_protein_id = 'NA';
	
	for (my $curr_ind = 0; $curr_ind < @header_parts; $curr_ind++)
	{
		my $curr_header_field = $header_parts[$curr_ind];
		print "curr_ind is: '$curr_ind' and curr_header_field is: '$curr_header_field'\n";
		if ($curr_header_field =~ m/[rR]eplicon[ _][aA]ccession/)
		{
			$ind_contig_id = $curr_ind;
			print "ind_contig_id is $ind_contig_id\n";
		}
		if ($curr_header_field =~ m/[pP]rotein[ _][pP]roduct/)
		{
			$ind_protein_id = $curr_ind;
			print "ind_protein_id is $ind_protein_id\n";
		}
	}
	
	if (($ind_contig_id eq 'NA') or ($ind_protein_id eq 'NA'))
	{
		die "the parsing of the NCBI protein table header didn't succeed. This is the header:\n$header\n";
	}
	
	
	my $line = <$in>;
	while (defined $line)
	{
		chomp ($line);
		my @line_parts = split(/\t/, $line);
		
		my $contig_id = $line_parts[$ind_contig_id];
		my $protein_id = $line_parts[$ind_protein_id];
		
		$contig_to_prot_ref->{$contig_id}{$protein_id} = 1;
		
		$line = <$in>;
	}
	
	close ($in);
}